package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.common.core.util.StringUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.mhfire.model.Alert;
import com.bizmatics.mhfire.persistence.mapper.AlertMapper;
import com.bizmatics.mhfire.service.AlertService;
import com.bizmatics.mhfire.service.util.Arith;
import com.bizmatics.mhfire.service.vo.AlertStatisticsVO;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author yq
 * @date 2021/5/28 14:53
 */
@Service
public class AlertServiceImpl extends AbstractCrudService<AlertMapper, Alert> implements AlertService {


    @Override
    public List<AlertStatisticsVO> getAlertStatisticsByHouse() {
        Date date = new Date();
        Date startTime = DateUtils.getDayStartTime(date);
        return enhanceList(baseMapper.getCountByHorse(startTime, date));
    }

    public List<AlertStatisticsVO> enhanceList(List<Map<String,Object>> mapList){
        List<AlertStatisticsVO> list = new ArrayList<>();
        for (Map<String, Object> map:mapList) {
            AlertStatisticsVO alertStatisticsVo = new AlertStatisticsVO();
            alertStatisticsVo.setType(map.get("aType").toString());
            alertStatisticsVo.setMonth(map.get("monthTime").toString());
            alertStatisticsVo.setNumber(Integer.parseInt(map.get("aCount").toString()));
            alertStatisticsVo.setType(map.get("aType").toString());
            list.add(alertStatisticsVo);
        }
        return list;
    }


    @Override
    public List<AlertStatisticsVO> getAlertStatisticsByMonth() {
        Date date = new Date();
        Date startTime = DateUtils.getDayStartTime(date);
        return enhanceList(baseMapper.getCountByMonth(startTime, date));
    }

    @Override
    public CommonPage<Alert> page(Integer current, Integer size, Date startTime, Date endTime) {
        IPage<Alert> page = new Page<>(current, size);
        LambdaQueryWrapper<Alert> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.between(Alert::getDcsj,startTime,endTime);
        page = this.page(page, queryWrapper);
        return  this.ToCommonPage(page);
    }

    @Override
    public List<AlertStatisticsVO> getAlertStatistics(Date startTime, Date endTime) {
        List<AlertStatisticsVO> list = new ArrayList<>();
        list.add(getAsV(startTime, endTime, "火灾"));
        list.add(getAsV(startTime, endTime, "社会救援"));
        list.add(getAsV(startTime, endTime, "抢险救援"));
        int sum = list.stream().mapToInt(AlertStatisticsVO::getNumber).sum();
        list.forEach(alertStatisticsVO -> {
            if (0 == sum){
                alertStatisticsVO.setRadio(0.00);
            }else {
                alertStatisticsVO.setRadio(Arith.div(alertStatisticsVO.getNumber(), sum));
            }
        });
        return list;
    }
    public AlertStatisticsVO getAsV(Date startTime,Date endTime,String type){
        AlertStatisticsVO alertStatisticsVo = new AlertStatisticsVO();
        LambdaQueryWrapper<Alert> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(StringUtils.isNotBlank(type),Alert::getAjlx,type);
        if (null != startTime && null != endTime){
            queryWrapper.between(Alert::getDcsj,startTime,endTime);
        }
        alertStatisticsVo.setType(type);
        alertStatisticsVo.setNumber(this.count(queryWrapper));
        return alertStatisticsVo;
    }

}
